!function(e,t,n){"use strict";t.module("ngCookies",["ng"]).factory("$cookies",["$rootScope","$browser",function(e,r){function i(){var e,i,o,u;for(e in a)l(s[e])&&r.cookies(e,n);for(e in s)i=s[e],t.isString(i)?i!==a[e]&&(r.cookies(e,i),u=!0):t.isDefined(a[e])?s[e]=a[e]:delete s[e];if(u){u=!1,o=r.cookies();for(e in s)s[e]!==o[e]&&(l(o[e])?delete s[e]:s[e]=o[e],u=!0)}}var o,s={},a={},u=!1,c=t.copy,l=t.isUndefined;return r.addPollFn(function(){var t=r.cookies();o!=t&&(o=t,c(t,a),c(t,s),u&&e.$apply())})(),u=!0,e.$watch(i),s}]).factory("$cookieStore",["$cookies",function(e){return{get:function(n){var r=e[n];return r?t.fromJson(r):r},put:function(n,r){e[n]=t.toJson(r)},remove:function(t){delete e[t]}}}])}(window,window.angular);